'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.auth = exports.authorization = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _bcryptjs = require('bcryptjs');

var _bcryptjs2 = _interopRequireDefault(_bcryptjs);

var _jwtSimple = require('jwt-simple');

var _jwtSimple2 = _interopRequireDefault(_jwtSimple);

var _ldapjs = require('ldapjs');

var _ldapjs2 = _interopRequireDefault(_ldapjs);

var _config = require('./../config/config');

var _config2 = _interopRequireDefault(_config);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var authorization = exports.authorization = function authorization(db) {
    return function (req, res, next) {
        var authorization = req.headers.authorization;
        if (!authorization) {
            return res.status(401).json({ error: 'No credentials sent!' });
        }
        var token = authorization.split(/Bearer\s?/)[1];
        var decoded = _jwtSimple2.default.decode(token, _config2.default.jwt_secret);
        if (!decoded) {
            res.status(401);
        }
        var User = db.user;

        User.findByPk(decoded.id, {
            include: [{ association: User.BusinessGroup }]
        }).then(function (user) {
            req.authenticated_user = user;
        }).catch(function (err) {
            console.log('error loading authenticated user', JSON.stringify(err));
        }).finally(function () {
            return next();
        });
    };
};

var auth = exports.auth = function auth(db) {
    return function (req, res, next) {
        var _req$body = req.body,
            login = _req$body.login,
            password = _req$body.password;

        var User = db.user;
        var Setting = db.setting;
        var UserRole = db.userRole;
        var UserType = db.userType;
        var Op = db.Sequelize.Op;

        User.findOne({
            where: { login: login },
            include: [User.BusinessGroup, User.Type, User.Role]
        }).then(function (user) {
            if (!user || user.type.name === 'ldap') {
                // try ldap search (bind)
                return Setting.findAll({
                    where: {
                        name: _defineProperty({}, Op.like, 'ldap\_%')
                    }
                })
                // save/update user (return Promise) or status 404 (res)
                .then(function (settings) {
                    // convert settings to options
                    var options = settings.reduce(function (a, s) {
                        a[s.name] = s.value;
                        return a;
                    }, {});

                    if (options.ldap_host) {
                        return ldapAuth(login, password, options);
                        // Gère les erreurs mais ne peut plus se connecter avec un user LDAP...
                        //return Promise(ldapAuth(login, password, options));
                    }
                }).then(function (ldapUser) {
                    console.log('ldap user found', ldapUser);
                    if (user) {
                        // console.log('update user', user);
                        return user.update(ldapUser);
                    }
                    // set role and type
                    return Promise.all([UserRole.findOne({
                        where: {
                            name: 'user'
                        }
                    }), UserType.findOne({
                        where: {
                            name: 'ldap'
                        }
                    })]).then(function (userSettings) {
                        // console.log('settings', userSettings);
                        var _userSettings = _slicedToArray(userSettings, 2),
                            userRole = _userSettings[0],
                            userType = _userSettings[1];

                        ldapUser.u_ur_id = userRole.id;
                        ldapUser.u_ut_id = userType.id;
                        console.log('create ldap user', ldapUser);
                        return User.create(ldapUser).then(function (created) {
                            return User.findByPk(created.id, {
                                include: [User.BusinessGroup, User.Type, User.Role]
                            });
                        });
                    }, function (err) {
                        return Promise.reject(err);
                    });
                });
            }

            // no ldap type user 
            return _bcryptjs2.default.compare(password, user.password).then(function (result) {
                if (!result) {
                    // res.status(404);
                    return Promise.reject();
                }

                return Promise.resolve(user);
            });
        }).then(function (user) {
            console.log('create user token...');
            // create JWT
            var payload = {
                login: user.login,
                role: user.role.name,
                id: user.id
            };
            var token = _jwtSimple2.default.encode(payload, _config2.default.jwt_secret);
            res.json(_extends({}, payload, {
                token: token
            }));
        }, function (err) {
            return res.status(404).json(err);
        }).catch(function (error) {
            return next(error);
        });
    };
};

var ldapAuth = function ldapAuth(login, password, options) {

    var clientOptions = {
        url: options.ldap_host + ':' + options.ldap_port,
        strictDN: true,
        reconnect: false,
        queueDisable: false,
        timeout: 10000,
        connectTimeout: 10000,
        idleTimeout: 10000,
        tlsOptions: {
            //ca: [options.ldap_tls_cert],
            rejectUnauthorized: [options.ldap_cert_validity],
            enableTrace: false
        }
    };

    console.log('create ldap client');
    var client = _ldapjs2.default.createClient(clientOptions);

    client.on('error', function (err) {
        console.error("An error is occured", err);
        //return ("An error is occured");
    });
    client.on('timeout', function (err) {
        console.error("Operation timing out");
        //return ("Operation timing out");
    });
    client.on('connectTimeout', function (err) {
        console.error("TCP connection timing out");
        //return ("TCP connection timing out");
    });
    client.on('idleTimeout', function (err) {
        console.info("Idle timing out");
        //return ("Idle timing out");
    });

    if (options.ldap_encrypt === 'TLS') {
        console.log('start TLS connection');
        return new Promise(function (resolve, reject) {
            client.starttls(clientOptions.tlsOptions, client.controls, function (err) {
                if (err) {
                    console.error('starttls error', JSON.stringify(err));
                    reject(err);
                    return;
                }
                resolve(ldapSearch(client, login, password, options));
            });
        });
    }
    return Promise.resolve(ldapSearch(client, login, password, options));
};

var ldapSearch = function ldapSearch(client, login, password, options) {

    var convertUser = function convertUser(ldapModel) {
        console.log('convert user', ldapModel);
        return {
            login: ldapModel[options.ldap_login_attribute],
            first_name: ldapModel[options.ldap_firstname_attribute],
            last_name: ldapModel[options.ldap_lastname_attribute],
            email: ldapModel[options.ldap_email_attribute]
        };
    };

    var ldapDnSearch = {
        filter: '(&(' + options.ldap_login_attribute + '=' + login + '))',
        scope: 'sub',
        attributes: ['dn', options.ldap_login_attribute, options.ldap_firstname_attribute, options.ldap_lastname_attribute, options.ldap_email_attribute]
    };

    return new Promise(function (resolve, reject) {
        console.log('bind user dn...');
        client.bind(options.ldap_bind_user, options.ldap_bind_password, function (err) {
            if (err) {
                console.error('bind error user dn');
                reject(err);
                return;
            }
            resolve();
        });
    }).then(function () {
        return new Promise(function (resolve, reject) {
            console.log('search user dn...');
            client.search(options.ldap_base_dn, ldapDnSearch, function (err, res) {
                var localEntry = null;
                if (err) {
                    console.error('search error');
                    reject(err);
                    return;
                };
                res.on('searchEntry', function (entry) {
                    // On se connecte (BIND) avec le 'dn' trouvé et le mot de passe fourni
                    // Si la connexion fonctionne, le mot de passe fourni est le bon
                    // Sinon le mot de passe est invalide (échec de connexion)
                    localEntry = entry.object;
                });
                res.on('end', function (result) {
                    console.log('search end with', localEntry);
                    localEntry ? resolve(localEntry) : reject(null);
                });
                res.on('error', function (err) {
                    console.error('search on error');
                    reject(err);
                });
            });
        });
    }, function (err) {
        throw new Error(err);
    }).then(function (ldapUser) {
        return new Promise(function (resolve, reject) {
            console.log('bind user dn...');
            client.bind(ldapUser.dn, password, function (err) {
                if (err) {
                    console.error('bind error user');
                    reject(err);
                    return;
                }
                client.unbind(function (err) {
                    if (err) {
                        console.error('unbind user error');
                    }
                    resolve(ldapUser);
                });
            });
        });
    }).then(function (ldapUser) {
        return new Promise(function (resolve) {
            client.unbind(function (err) {
                if (err) {
                    console.error('unbind user dn error');
                }
                resolve(convertUser(ldapUser));
            });
        });
    }, function (err) {
        return Promise.reject(err);
    });
};