'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _resourceRouterMiddleware = require('resource-router-middleware');

var _resourceRouterMiddleware2 = _interopRequireDefault(_resourceRouterMiddleware);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var filterUserInfo = function filterUserInfo(db) {
	return {
		include: [{
			association: db.businessGroup.User,
			attributes: {
				exclude: ['password']
			}
		}]
	};
};

exports.default = function (_ref) {
	var config = _ref.config,
	    db = _ref.db;
	return (0, _resourceRouterMiddleware2.default)({

		/** Property name to store preloaded entity on `request`. */
		id: 'businessGroup',

		/** For requests with an `id`, you can auto-load the entity.
   *  Errors terminate the request, success sets `req[id] = data`.
   */
		load: function load(req, id, callback) {
			var BusinessGroup = db.businessGroup;

			BusinessGroup.findByPk(id, filterUserInfo(db)).then(function (business) {
				return callback(null, business);
			}).catch(function (err) {
				return callback(console.log(err));
			});
		},


		/** GET / - List all entities */
		list: function list(_ref2, res) {
			var params = _ref2.params;

			var BusinessGroup = db.businessGroup;
			BusinessGroup.findAll(_extends({
				order: [['name', 'ASC']]
			}, filterUserInfo(db))).then(function (business) {
				return res.json(business);
			}).catch(function (err) {
				return res.status(500).json({ error: err });
			});
		},


		/** POST / - Create a new entity */
		create: function create(_ref3, res) {
			var body = _ref3.body;

			var BusinessGroup = db.businessGroup;

			var business = _extends({}, body, {
				users: body.users.map(function (u) {
					return u.id;
				})
			});

			var model = BusinessGroup.build(business);
			model.save().then(function (created) {
				return created.setUsers(business.users).then(function () {
					return BusinessGroup.findByPk(created.id, filterUserInfo(db));
				});
			}).then(function (b) {
				return res.json(b);
			}).catch(function (err) {
				return res.status(500).json({ error: err });
			});
		},


		/** GET /:id - Return a given entity */
		read: function read(_ref4, res) {
			var businessGroup = _ref4.businessGroup;

			res.json(businessGroup);
		},


		/** PUT /:id - Update a given entity */
		update: function update(_ref5, res) {
			var businessGroup = _ref5.businessGroup,
			    body = _ref5.body;

			db.sequelize.transaction(function (t) {
				return businessGroup.update(body, { transaction: t }).then(function () {
					return businessGroup.setUsers(body.users.map(function (u) {
						return u.id;
					}), { transaction: t });
				});
			}).then(function () {
				return businessGroup.reload(filterUserInfo(db));
			}).then(function (updated) {
				return res.json(updated);
			}).catch(function (err) {
				return res.status(500).json({ error: err });
			});
		},


		/** DELETE /:id - Delete a given entity */
		delete: function _delete(_ref6, res) {
			var businessGroup = _ref6.businessGroup;

			db.sequelize.transaction(function (t) {
				return businessGroup.setUsers([], { transaction: t }).then(function () {
					return businessGroup.destroy({ transaction: t });
				});
			}).then(function () {
				return res.sendStatus(204);
			}).catch(function (err) {
				return res.status(500).json({ error: err });
			});
		}
	});
};