'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _resourceRouterMiddleware = require('resource-router-middleware');

var _resourceRouterMiddleware2 = _interopRequireDefault(_resourceRouterMiddleware);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

exports.default = function (_ref) {
	var config = _ref.config,
	    db = _ref.db;
	return (0, _resourceRouterMiddleware2.default)({

		/** Property name to store preloaded entity on `request`. */
		id: 'contributor',

		/** For requests with an `id`, you can auto-load the entity.
   *  Errors terminate the request, success sets `req[id] = data`.
   */
		load: function load(req, id, callback) {
			var Contributor = db.contributor;

			Contributor.findByPk(id, { include: [Contributor.Vigilance] }).then(function (contributor) {
				return callback(null, contributor);
			}).catch(function (err) {
				return callback(console.log(err));
			});
		},


		/** GET / - List all entities */
		list: function list(_ref2, res) {
			var params = _ref2.params;

			var Contributor = db.contributor;
			Contributor.findAll({
				include: [Contributor.Vigilance],
				order: [['auto_validation', 'DESC'], ['internal', 'DESC'], ['name', 'ASC']]
			}).then(function (contributors) {
				return res.json(contributors);
			}).catch(function (err) {
				return console.log(err);
			});
		},


		/** POST / - Create a new entity */
		create: function create(_, res) {
			res.status(403);
		},


		/** GET /:id - Return a given entity */
		read: function read(_ref3, res) {
			var contributor = _ref3.contributor;

			res.json(contributor);
		},


		/** PUT /:id - Update a given entity */
		update: function update(_ref4, res) {
			var contributor = _ref4.contributor,
			    body = _ref4.body;

			var Op = db.Sequelize.Op;
			var Vigilance = db.vigilance;

			var vigilanceFilter = body.internal ? { name: 'internal' } : { range: _defineProperty({}, Op.contains, body.score) };

			db.sequelize.transaction(function (t) {
				return Vigilance.findOne({ where: vigilanceFilter }, { transaction: t }).then(function (vigilance) {
					return contributor.update(body, { transaction: t }).then(function () {
						return contributor.setVigilance(vigilance, { transaction: t });
					});
				});
			}).then(function () {
				return contributor.reload();
			}).then(function (updated) {
				return res.json(updated);
			}).catch(function (err) {
				return res.status(500).json({ error: err });
			});
		},


		/** DELETE /:id - Delete a given entity */
		delete: function _delete(_, res) {
			res.status(403);
		}
	});
};