'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _resourceRouterMiddleware = require('resource-router-middleware');

var _resourceRouterMiddleware2 = _interopRequireDefault(_resourceRouterMiddleware);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var filterUserInfo = function filterUserInfo(db) {
	return {
		include: [{
			association: db.footprint.User,
			attributes: {
				exclude: ['password']
			}
		}]
	};
};

/**
 * Supprime les enregistrements liés à l'emprise
 */
function deleteFootprintRelations(seq, id) {
	var sRequest = '\n\t\tDELETE FROM lebontag.lbt_osmdiff_footprint WHERE fp_id = ' + id + ';\n\t';
	return seq.query(sRequest);
}

/**
 * Calcule les intersections liées à l'emprise
 */
function calculateFootprintRelations(seq, id) {
	var sRequest = '\n    DROP INDEX IF EXISTS lebontag.osmdiff_footprint_odidv_idx;\n    DROP INDEX IF EXISTS lebontag.osmdiff_footprint_odid_idx;\n    DROP INDEX IF EXISTS lebontag.osmdiff_footprint_fpid_idx;\n\n    WITH footprints AS ( SELECT fp_id,(st_dump(ST_CollectionExtract(ST_MakeValid(fp_geom),3))).geom FROM lebontag.lbt_footprint WHERE fp_id = ' + id + ' ) INSERT INTO lebontag.lbt_osmdiff_footprint (od_id,od_new_version,fp_id) SELECT d.od_id,d.od_new_version,f.fp_id FROM lebontag.lbt_osmdiff AS d INNER JOIN footprints AS f ON ST_Intersects(d.od_new_geom, f.geom) WHERE d.od_new_geom IS NOT NULL UNION SELECT d.od_id,d.od_new_version,f.fp_id FROM lebontag.lbt_osmdiff AS d INNER JOIN footprints AS f ON ST_Intersects(d.od_old_geom, f.geom) WHERE d.od_old_geom IS NOT NULL;\n\n    CREATE INDEX osmdiff_footprint_odidv_idx ON lebontag.lbt_osmdiff_footprint USING btree (od_id,od_new_version);\n    CREATE INDEX osmdiff_footprint_odid_idx ON lebontag.lbt_osmdiff_footprint USING btree (od_id);\n    CREATE INDEX osmdiff_footprint_fpid_idx ON lebontag.lbt_osmdiff_footprint USING btree (fp_id);\n\t';
	return seq.query(sRequest);
}

/**
 * Retourne une promesse de l'emprise par défaut
 * @param {*} db 
 * @returns 
 */
function getDefaultFootprint(db) {
	var Setting = db.setting;
	return new Promise(function (resolve, reject) {
		Setting.findAll({
			where: {
				s_name: 'osm_footprint'
			}
		}).then(function (res) {
			var setting = res[0];
			if (setting && setting.dataValues && setting.dataValues.value) {
				resolve(setting.dataValues.value);
			} else {
				resolve(null);
			}
		}).catch(function (err) {
			console.error(err);
			reject();
		});
	});
}

/**
 * Transforme un GeoJSON pour être interprété par PostGIS
 * @param {*} geojson 
 * @returns 
 */
function geomFromGeoJSON(geojson) {
	if (geojson && geojson.type === 'FeatureCollection' && geojson.features && geojson.features[0] && geojson.features[0].geometry) {

		// Cas de FeatureCollection 
		// la fonction ST_GeomFromGeoJSON utilisée par sequelize ne prend pas en compte les FeatureCollection
		// alors il faut créer une GeometryCollection
		var geomCollection = {
			type: 'GeometryCollection',
			geometries: []
		};
		for (var i = 0; i < geojson.features.length; i++) {
			if (geojson.features[i].geometry) {
				geomCollection.geometries.push(geojson.features[i].geometry);
			}
		}

		return geomCollection;
	} else {

		// Cas de simple géométrie
		return geojson;
	}
}

exports.default = function (_ref) {
	var config = _ref.config,
	    db = _ref.db;
	return (0, _resourceRouterMiddleware2.default)({

		/** Property name to store preloaded entity on `request`. */
		id: 'footprint',

		/** For requests with an `id`, you can auto-load the entity.
   *  Errors terminate the request, success sets `req[id] = data`.
   */
		load: function load(req, id, callback) {
			var Footprint = db.footprint;

			getDefaultFootprint(db).then(function (defaultFootprintId) {
				Footprint.findByPk(id, filterUserInfo(db)).then(function (footprint) {
					if (footprint && footprint.dataValues && footprint.dataValues.id == Number(defaultFootprintId)) {
						footprint.dataValues.default = true;
					} else {
						footprint.dataValues.default = false;
					}
					callback(null, footprint);
				}).catch(function (err) {
					return callback(console.error(err));
				});
			});
		},


		/** GET /:id - Return a given entity */
		read: function read(_ref2, res) {
			var footprint = _ref2.footprint;

			res.json(footprint);
		},


		/** GET / - List all entities */
		list: function list(_ref3, res) {
			var params = _ref3.params;

			var Footprint = db.footprint;

			getDefaultFootprint(db).then(function (defaultFootprintId) {
				Footprint.findAll(_extends({
					order: [['name', 'ASC']]
				}, filterUserInfo(db))).then(function (footprints) {
					for (var i = 0; i < footprints.length; i++) {
						if (footprints[i].dataValues) {
							if (footprints[i].dataValues.id == Number(defaultFootprintId)) {
								footprints[i].dataValues.default = true;
							} else {
								footprints[i].dataValues.default = false;
							}
						}
					}
					res.json(footprints);
				}).catch(function (err) {
					return res.status(500).json({ error: err });
				});
			}).catch(function (err) {
				return res.status(500).json({ error: err });
			});
		},


		/** POST / - Create a new entity */
		create: function create(_ref4, res) {
			var body = _ref4.body;

			var Footprint = db.footprint;

			var footprint = _extends({}, body, {
				users: body.users.map(function (u) {
					return u.id;
				})
			});

			if (footprint.geojson) {
				footprint.geom = geomFromGeoJSON(footprint.geojson);
			}

			var model = Footprint.build(footprint);
			model.save().then(function (created) {
				return created.setUsers(footprint.users).then(function () {
					return Footprint.findByPk(created.id, filterUserInfo(db));
				});
			}).then(function (b) {
				return res.json(b);
			}).catch(function (err) {
				return res.status(500).json({ error: err });
			});
		},


		/** PUT /:id - Update a given entity */
		update: function update(_ref5, res) {
			var footprint = _ref5.footprint,
			    body = _ref5.body;


			// Ne met pas à jour le JSON s'il n'a pas été modifié
			if (body.geojson) {
				body.geom = geomFromGeoJSON(body.geojson);
			} else {
				delete body.geom;
			}

			db.sequelize.transaction(function (t) {
				return footprint.update(body, { transaction: t }).then(function () {
					return footprint.setUsers(body.users.map(function (u) {
						return u.id;
					}), { transaction: t });
				});
			}).then(function () {
				return footprint.reload(filterUserInfo(db));
			}).then(function (updated) {
				return res.json(updated);
			}).catch(function (err) {
				return res.status(500).json({ error: err });
			});
		},


		/** DELETE /:id - Delete a given entity */
		delete: function _delete(_ref6, res) {
			var footprint = _ref6.footprint;

			if (footprint.dataValues && footprint.dataValues.default !== true) {
				db.sequelize.transaction(function (t) {
					return footprint.setUsers([], { transaction: t }).then(function () {
						return footprint.destroy({ transaction: t });
					});
				}).then(function () {
					return res.sendStatus(204);
				}).catch(function (err) {
					return res.status(500).json({ error: err });
				});
			} else {
				res.status(500).json({ error: 'Cannot delete default printfoot' });
			}
		}
	})

	/** PUT /:id/clean : delete rows in lbt_osmdiff_footprint */
	.put('/:id/clean', function (req, res, next) {
		var seq = db.sequelize;
		return deleteFootprintRelations(seq, req.params.id).then(function () {
			res.json("Clean ok");
			//res.sendStatus(204);
		}).catch(function (err) {
			console.error('Delete footprint relations error', err);
			// Rollback transaction
			t.rollback();
			// Retour erreur 500
			res.status(500).json({
				error: err
			});
		});
	})

	/** PUT /:id/calculate : calculate new intersections in lbt_osmdiff_footprint */
	.put('/:id/calculate', function (req, res, next) {
		var seq = db.sequelize;
		return calculateFootprintRelations(seq, req.params.id).then(function () {
			res.json("Calculate ok");
			//res.sendStatus(204);
		}).catch(function (err) {
			console.error('Calculate new intersections error', err);
			// Rollback transaction
			t.rollback();
			// Retour erreur 500
			res.status(500).json({
				error: err
			});
		});
	});
};