'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _express = require('express');

var _util = require('../lib/util');

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function getThemes(db, user) {

	var Theme = db.theme;
	var ObjectGroup = db.objectGroup;
	var Right = db.right;
	var BusinessGroup = db.businessGroup;
	var User = db.user;
	var Op = db.Sequelize.Op;

	return new Promise(function (resolve, reject) {

		Theme.findAll({
			where: {
				'$objects.rights.business.users.u_id$': _defineProperty({}, Op.eq, user.id),
				// Ne s'occupe que des groupes d'objets actifs
				'$objects.og_active$': _defineProperty({}, Op.eq, true),
				// Ne s'occupe pas des groupes d'objets en auto-validation
				'$objects.og_auto$': _defineProperty({}, Op.ne, true)
			},
			order: [['name', 'ASC'], [Theme.ObjectGroup, 'name', 'ASC']],
			include: [{
				model: ObjectGroup,
				as: 'objects',
				include: [{
					association: ObjectGroup.Keys
				}, {
					model: Right,
					as: 'rights',
					include: [{
						association: Right.Keys
					}, {
						model: BusinessGroup,
						as: 'business',
						include: [{
							model: User,
							as: 'users',
							// only attribute for restrict results without loading User
							attributes: ['u_id']
						}]
					}]
				}]
			}]
		}).then(function (themes) {
			resolve(themes);
		}).catch(function (err) {
			reject(err);
		});
	});
}

exports.default = function (_ref) {
	var config = _ref.config,
	    db = _ref.db;

	var map = (0, _express.Router)();

	map.get('/menu', function (req, res, next) {

		var user = req.authenticated_user;

		// Thèmes associés à l'utilisateur
		getThemes(db, user).then(function (themes) {

			// Filtre
			var filter = {};
			if (req.query && req.query.filter) {
				try {
					filter = JSON.parse(req.query.filter);
				} catch (error) {}
			}

			(0, _util.getThemesOsmDiff)(db, themes, req.query.footprint, filter).then(function (tree) {
				return res.json(tree);
			}).catch(function (err) {
				res.status(500).json({
					error: err
				});
			});
		}).catch(function (err) {
			res.status(500).json({
				error: err
			});
		});
	});

	map.get('/menu/object/:id', function (req, res, next) {
		var Right = db.right;
		// const Tag = db.tag;
		var ObjectGroup = db.objectGroup;

		Right.findByPk(req.params.id, {
			include: [{
				association: Right.Object,
				// model: ObjectGroup,
				// as: 'object',
				include: [{
					association: ObjectGroup.Keys
				}]
			}, {
				association: Right.Keys
				// model: Tag,
				// as: 'keys',
			}]
		}).then(function (right) {
			var obj = right.object.toJSON();
			var sequelize = db.sequelize;
			var Op = db.Sequelize.Op;

			var hstoreContainHstore = function hstoreContainHstore(col, hArray) {
				return sequelize.where(sequelize.col(col), '@>', sequelize.fn('hstore', sequelize.literal('ARRAY[\'' + hArray.join('\',\'') + '\']')));
			};

			var hstoreContainKeys = function hstoreContainKeys(col, op, keys) {
				return sequelize.where(sequelize.col(col), op, sequelize.literal('ARRAY[\'' + keys.join('\',\'') + '\']'));
			};

			var getClauses = function getClauses(col) {
				var clauses = [];

				if (Object.keys(obj.query || {}).length) {
					var query = Object.entries(obj.query).reduce(function (a, _ref2) {
						var _ref3 = _slicedToArray(_ref2, 2),
						    k = _ref3[0],
						    v = _ref3[1];

						return a.concat(k, v);
					}, []);
					// @> hstore(ARRAY[:query])
					clauses.push(hstoreContainHstore(col, query));
				}
				if (obj.keys.length) {
					var keys = obj.keys.map(function (k) {
						return k.name;
					});
					// ?& ARRAY[:keys]
					clauses.push(hstoreContainKeys(col, '?&', keys));
				}
				if (right.getKeys().length) {
					var _keys = right.getKeys().map(function (k) {
						return k.name;
					});
					// ?| ARRAY[:keys]
					clauses.push(hstoreContainKeys(col, '?|', _keys));
				}
				return clauses;
			};

			var newClauses = getClauses('od_new_tags');
			var oldClauses = getClauses('od_old_tags');

			return (
				//newClauses.length ?
				db.osmdiff.findAll({
					// attributes: {
					// 	include: [
					// 		[db.sequelize.fn('ST_Transform', db.sequelise.col('od_new_geom'), 4326), 'od_new_geom'],
					// 		[db.sequelize.fn('ST_Transform', db.sequelise.col('od_old_geom'), 4326), 'od_old_geom'],
					// 	],
					// },
					where: _defineProperty({}, Op.or, [_defineProperty({}, Op.and, [].concat(_toConsumableArray(newClauses), [{
						action: ['create', 'modify']
					}])), _defineProperty({}, Op.and, [].concat(_toConsumableArray(oldClauses), [{
						action: 'delete'
					}]))]),
					include: [{
						association: db.osmdiff.NewChangeset
					}, {
						model: db.contributor,
						as: 'contributor',
						include: [{
							association: db.contributor.Vigilance
						}]
					}]
				})
				//: Promise.resolve([])
				.then(function (diff) {
					return Promise.resolve(_extends({}, obj, {
						osmdiff: diff.map(function (d) {
							return d.toJSON();
						})
					}));
				}).catch(function (err) {
					console.log('err', err);
					throw err;
				})
			);
		}).then(function (object) {
			return res.json(object);
		}).catch(function (err) {
			return res.status(500).json({
				error: err
			});
		});
	});

	return map;
};