'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _resourceRouterMiddleware = require('resource-router-middleware');

var _resourceRouterMiddleware2 = _interopRequireDefault(_resourceRouterMiddleware);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = function (_ref) {
	var config = _ref.config,
	    db = _ref.db;
	return (0, _resourceRouterMiddleware2.default)({

		/** Property name to store preloaded entity on `request`. */
		id: 'objectGroup',

		/** For requests with an `id`, you can auto-load the entity.
   *  Errors terminate the request, success sets `req[id] = data`.
   */
		load: function load(req, id, callback) {
			var ObjectGroup = db.objectGroup;

			ObjectGroup.findByPk(id, { include: [ObjectGroup.Theme, ObjectGroup.Keys] }).then(function (object) {
				return callback(null, object);
			}).catch(function (err) {
				return callback(console.log(err));
			});
		},


		/** GET / - List all entities */
		list: function list(_ref2, res) {
			var params = _ref2.params,
			    query = _ref2.query;

			var ObjectGroup = db.objectGroup;
			ObjectGroup.scope(query.scope).findAll({
				order: [['name', 'ASC']],
				include: [ObjectGroup.Theme, ObjectGroup.Keys]
			}).then(function (objects) {
				return res.json(objects);
			}).catch(function (err) {
				return res.status(500).json({ error: err });
			});
		},


		/** POST / - Create a new entity */
		create: function create(_ref3, res) {
			var body = _ref3.body;

			var ObjectGroup = db.objectGroup;

			var model = ObjectGroup.build(body);
			model.save().then(function (created) {
				return created.setTheme(body.theme.id).then(function () {
					return created.setKeys(body.keys.map(function (k) {
						return k.id;
					})).then(function () {
						return ObjectGroup.findByPk(created.id, { include: [ObjectGroup.Theme, ObjectGroup.Keys] });
					});
				});
			}).then(function (o) {
				return res.json(o);
			}).catch(function (err) {
				return res.status(500).json({ error: err });
			});
		},


		/** GET /:id - Return a given entity */
		read: function read(_ref4, res) {
			var objectGroup = _ref4.objectGroup;

			res.json(objectGroup);
		},


		/** PUT /:id - Update a given entity */
		update: function update(_ref5, res) {
			var objectGroup = _ref5.objectGroup,
			    body = _ref5.body;

			var ObjectGroup = db.objectGroup;

			db.sequelize.transaction(function (t) {
				return objectGroup.update(body, { transaction: t }).then(function () {
					return objectGroup.setTheme(body.theme.id, { transaction: t }).then(function () {
						return objectGroup.setKeys(body.keys.map(function (k) {
							return k.id;
						}), { transaction: t });
					});
				});
			}).then(function () {
				return objectGroup.reload({ include: [ObjectGroup.Theme, ObjectGroup.Keys] });
			}).then(function (updated) {
				return res.json(updated);
			}).catch(function (err) {
				return res.status(500).json({ error: err });
			});
		},


		/** DELETE /:id - Delete a given entity */
		delete: function _delete(_ref6, res) {
			var objectGroup = _ref6.objectGroup;

			db.sequelize.transaction(function (t) {
				return objectGroup.setKeys([], { transaction: t }).then(function () {
					return objectGroup.destroy({ transaction: t });
				});
			}).then(function () {
				return res.sendStatus(204);
			}).catch(function (err) {
				return res.status(500).json({ error: err });
			});
		}
	});
};