'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _resourceRouterMiddleware = require('resource-router-middleware');

var _resourceRouterMiddleware2 = _interopRequireDefault(_resourceRouterMiddleware);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = function (_ref) {
	var config = _ref.config,
	    db = _ref.db;
	return (0, _resourceRouterMiddleware2.default)({

		/** Property name to store preloaded entity on `request`. */
		id: 'right',

		/** For requests with an `id`, you can auto-load the entity.
   *  Errors terminate the request, success sets `req[id] = data`.
   */
		load: function load(req, id, callback) {
			var Right = db.right;

			Right.findByPk(id, { include: [Right.Business, Right.Object, Right.Keys] }).then(function (right) {
				return callback(null, right);
			}).catch(function (err) {
				return callback(console.log(err));
			});
		},


		/** GET / - List all entities */
		list: function list(_ref2, res) {
			var params = _ref2.params;

			var Right = db.right;

			Right.findAll({
				order: [[Right.Business, 'name', 'ASC'], [Right.Object, 'name', 'ASC']],
				include: [Right.Business, Right.Object, Right.Keys]
			}).then(function (rights) {
				return res.json(rights);
			}).catch(function (err) {
				return res.status(500).json({ error: err });
			});
		},


		/** POST / - Create a new entity */
		create: function create(_ref3, res) {
			var body = _ref3.body;

			var Right = db.right;

			var model = Right.build(body);
			model.bg_id = body.business.id;
			model.og_id = body.object.id;
			model.save().then(function (created) {
				return created.setKeys(body.keys.map(function (k) {
					return k.id;
				})).then(function () {
					return Right.findByPk(created.id, { include: [Right.Business, Right.Object, Right.Keys] });
				});
			}).then(function (r) {
				return res.json(r);
			}).catch(function (err) {
				return res.status(500).json({ error: err });
			});
		},


		/** GET /:id - Return a given entity */
		read: function read(_ref4, res) {
			var right = _ref4.right;

			res.json(right);
		},


		/** PUT /:id - Update a given entity */
		update: function update(_ref5, res) {
			var right = _ref5.right,
			    body = _ref5.body;

			body.bg_id = body.business.id;
			body.og_id = body.object.id;
			db.sequelize.transaction(function (t) {
				return right.update(body, { transaction: t }).then(function () {
					return right.setKeys(body.keys.map(function (k) {
						return k.id;
					}), { transaction: t });
				});
			}).then(function () {
				return right.reload();
			}).then(function (updated) {
				return res.json(updated);
			}).catch(function (err) {
				return res.status(500).json({ error: err });
			});
		},


		/** DELETE /:id - Delete a given entity */
		delete: function _delete(_ref6, res) {
			var right = _ref6.right;

			db.sequelize.transaction(function (t) {
				return right.setKeys([], { transaction: t }).then(function () {
					return right.destroy({ transaction: t });
				});
			}).then(function () {
				return res.sendStatus(204);
			}).catch(function (err) {
				return res.status(500).json({ error: err });
			});
		}
	});
};