'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _resourceRouterMiddleware = require('resource-router-middleware');

var _resourceRouterMiddleware2 = _interopRequireDefault(_resourceRouterMiddleware);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = function (_ref) {
	var config = _ref.config,
	    db = _ref.db;
	return (0, _resourceRouterMiddleware2.default)({

		/** Property name to store preloaded entity on `request`. */
		id: 'setting',

		/** For requests with an `id`, you can auto-load the entity.
   *  Errors terminate the request, success sets `req[id] = data`.
   */
		load: function load(req, id, callback) {
			var Setting = db.setting;
			Setting.findByPk(id).then(function (setting) {
				return callback(null, setting);
			}).catch(function (err) {
				return callback(console.log(err));
			});
		},


		/** GET / - List all entities */
		list: function list(_ref2, res) {
			var params = _ref2.params;

			var Setting = db.setting;

			Setting.findAll().then(function (settings) {
				return res.json(settings);
			}).catch(function (err) {
				return console.log(err);
			});
		},


		/** POST / - Create a new entity */
		create: function create(_ref3, res) {
			var body = _ref3.body;

			var Setting = db.setting;

			Setting.create(body).then(function (setting) {
				return res.json(setting);
			}).catch(function (err) {
				return res.status(403).json({ error: JSON.stringify(err) });
			});
		},


		/** GET /:id - Return a given entity */
		read: function read(_ref4, res) {
			var setting = _ref4.setting;

			res.json(setting);
		},


		/** PUT /:id - Update a given entity */
		update: function update(_ref5, res) {
			var setting = _ref5.setting,
			    body = _ref5.body;

			setting.update(body).then(function () {
				return setting.reload();
			}).then(function (updated) {
				return res.json(updated);
			}).catch(function (err) {
				return res.status(500).json({ error: err });
			});
		},


		/** DELETE /:id - Delete a given entity */
		delete: function _delete(_ref6, res) {
			var setting = _ref6.setting;

			setting.destroy().then(function () {
				return res.status(204);
			}).catch(function (err) {
				return res.status(500).json({ error: err });
			});
		}
	});
};