'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _resourceRouterMiddleware = require('resource-router-middleware');

var _resourceRouterMiddleware2 = _interopRequireDefault(_resourceRouterMiddleware);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = function (_ref) {
	var config = _ref.config,
	    db = _ref.db;
	return (0, _resourceRouterMiddleware2.default)({

		/** Property name to store preloaded entity on `request`. */
		id: 'tag',

		/** For requests with an `id`, you can auto-load the entity.
   *  Errors terminate the request, success sets `req[id] = data`.
   */
		load: function load(req, id, callback) {
			var Tag = db.tag;
			Tag.findByPk(id).then(function (tag) {
				return callback(null, tag);
			}).catch(function (err) {
				return callback(console.log(err));
			});
		},


		/** GET / - List all entities */
		list: function list(_ref2, res) {
			var params = _ref2.params;

			var Tag = db.tag;

			var filter = { where: { deleted: false } };
			Tag.findAll(_extends({}, filter, {
				order: [['name', 'ASC']]
			})).then(function (tags) {
				return res.json(tags);
			}).catch(function (err) {
				return console.log(err);
			});
		},


		/** POST / - Create a new entity */
		create: function create(_ref3, res) {
			var body = _ref3.body;

			var Tag = db.tag;

			Tag.findOne({ where: { name: body.name } }).then(function (tag) {
				if (tag) {
					if (!tag.deleted) {
						return new Promise.reject("Le tag existe déjà et n'est pas supprimé");
					}
					return tag.update({
						name_fr: body.name_fr,
						deleted: false
					});
				}
				return Tag.create(body);
			}).then(function (tag) {
				return res.json(tag);
			}).catch(function (err) {
				return res.status(403).json({ error: JSON.stringify(err) });
			});
		},


		/** GET /:id - Return a given entity */
		read: function read(_ref4, res) {
			var tag = _ref4.tag;

			res.json(tag);
		},


		/** PUT /:id - Update a given entity */
		update: function update(_ref5, res) {
			var tag = _ref5.tag,
			    body = _ref5.body;

			tag.update(body).then(function () {
				return tag.reload();
			}).then(function (updated) {
				return res.json(updated);
			}).catch(function (err) {
				return res.status(500).json({ error: err });
			});
		},


		/** DELETE /:id - Delete a given entity */
		delete: function _delete(_ref6, res) {
			var tag = _ref6.tag;

			tag.update({ deleted: true }).then(function () {
				return tag.reload();
			}).then(function (updated) {
				return res.json(updated);
			}).catch(function (err) {
				return res.status(500).json({ error: err });
			});
		}
	});
};