'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _resourceRouterMiddleware = require('resource-router-middleware');

var _resourceRouterMiddleware2 = _interopRequireDefault(_resourceRouterMiddleware);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = function (_ref) {
	var config = _ref.config,
	    db = _ref.db;
	return (0, _resourceRouterMiddleware2.default)({

		/** Property name to store preloaded entity on `request`. */
		id: 'theme',

		/** For requests with an `id`, you can auto-load the entity.
   *  Errors terminate the request, success sets `req[id] = data`.
   */
		load: function load(req, id, callback) {
			var Theme = db.theme;

			Theme.findByPk(id, { include: [Theme.ObjectGroup] }).then(function (theme) {
				return callback(null, theme);
			}).catch(function (err) {
				return callback(console.log(err));
			});
		},


		/** GET / - List all entities */
		list: function list(_ref2, res) {
			var params = _ref2.params;

			var Theme = db.theme;
			Theme.findAll({
				order: [['name', 'ASC']],
				include: [Theme.ObjectGroup]
			}).then(function (themes) {
				return res.json(themes);
			}).catch(function (err) {
				return console.log(err);
			});
		},


		/** POST / - Create a new entity */
		create: function create(_ref3, res) {
			var body = _ref3.body;

			var Theme = db.theme;

			var theme = _extends({}, body, {
				objects: body.objects.map(function (o) {
					return o.id;
				})
			});

			var model = Theme.build(theme);
			model.save().then(function (created) {
				return created.setObjects(theme.objects).then(function () {
					return Theme.findByPk(created.id, { include: [Theme.ObjectGroup] });
				});
			}).then(function (t) {
				return res.json(t);
			}).catch(function (err) {
				return res.status(500).json({ error: err });
			});
		},


		/** GET /:id - Return a given entity */
		read: function read(_ref4, res) {
			var theme = _ref4.theme;

			res.json(theme);
		},


		/** PUT /:id - Update a given entity */
		update: function update(_ref5, res) {
			var theme = _ref5.theme,
			    body = _ref5.body;

			var Theme = db.theme;

			db.sequelize.transaction(function (t) {
				return theme.update(body, { transaction: t }).then(function () {
					return theme.setObjects(body.objects.map(function (o) {
						return o.id;
					}), { transaction: t });
				});
			}).then(function () {
				return theme.reload({ include: [Theme.ObjectGroup] });
			}).then(function (updated) {
				return res.json(updated);
			}).catch(function (err) {
				return res.status(500).json({ error: err });
			});
		},


		/** DELETE /:id - Delete a given entity */
		delete: function _delete(_ref6, res) {
			var theme = _ref6.theme;

			db.sequelize.transaction(function (t) {
				return theme.setObjects([], { transaction: t }).then(function () {
					return theme.destroy({ transaction: t });
				});
			}).then(function () {
				return res.sendStatus(204);
			}).catch(function (err) {
				return res.status(500).json({ error: err });
			});
		}
	});
};