'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _resourceRouterMiddleware = require('resource-router-middleware');

var _resourceRouterMiddleware2 = _interopRequireDefault(_resourceRouterMiddleware);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = function (_ref) {
	var config = _ref.config,
	    db = _ref.db;
	return (0, _resourceRouterMiddleware2.default)({

		/** Property name to store preloaded entity on `request`. */
		id: 'tile',

		/** For requests with an `id`, you can auto-load the entity.
   *  Errors terminate the request, success sets `req[id] = data`.
   */
		load: function load(req, id, callback) {
			var Tile = db.tile;

			Tile.findByPk(id).then(function (tile) {
				return callback(null, tile);
			}).catch(function (err) {
				return callback(console.log(err));
			});
		},


		/** GET / - List all entities */
		list: function list(_ref2, res) {
			var params = _ref2.params;

			var Tile = db.tile;

			Tile.findAll({
				order: [['name', 'ASC']]
			}).then(function (tile) {
				return res.json(tile);
			}).catch(function (err) {
				return res.status(500).json({ error: err });
			});
		},


		/** POST / - Create a new entity */
		create: function create(_ref3, res) {
			var body = _ref3.body;

			var Tile = db.tile;
			Tile.create(body).then(function (tile) {
				return res.json(tile);
			}).catch(function (err) {
				return res.status(500).json({ error: err });
			});
		},


		/** GET /:id - Return a given entity */
		read: function read(_ref4, res) {
			var tag = _ref4.tag;

			res.json(tag);
		},


		/** PUT /:id - Update a given entity */
		update: function update(_ref5, res) {
			var tile = _ref5.tile,
			    body = _ref5.body;

			tile.update(body).then(function () {
				return tile.reload();
			}).then(function (updated) {
				return res.json(updated);
			}).catch(function (err) {
				return res.status(500).json({ error: err });
			});
		},


		/** DELETE /:id - Delete a given entity */
		delete: function _delete(_ref6, res) {
			var tile = _ref6.tile;

			tile.destroy().then(function () {
				return res.sendStatus(204);
			}).catch(function (err) {
				return res.status(500).json({ error: err });
			});
		}
	});
};