'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _resourceRouterMiddleware = require('resource-router-middleware');

var _resourceRouterMiddleware2 = _interopRequireDefault(_resourceRouterMiddleware);

var _bcryptjs = require('bcryptjs');

var _bcryptjs2 = _interopRequireDefault(_bcryptjs);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var attributeFilter = { exclude: ['password'] };
var saltRounds = 10;

exports.default = function (_ref) {
	var config = _ref.config,
	    db = _ref.db;
	return (0, _resourceRouterMiddleware2.default)({

		/** Property name to store preloaded entity on `request`. */
		id: 'user',

		/** For requests with an `id`, you can auto-load the entity.
   *  Errors terminate the request, success sets `req[id] = data`.
   */
		load: function load(req, id, callback) {
			var User = db.user;

			User.findByPk(id, {
				include: [User.BusinessGroup, User.Type, User.Role, User.Footprint],
				attributes: attributeFilter
			}).then(function (user) {
				return callback(null, user);
			}).catch(function (err) {
				return callback(console.log(err));
			});
		},


		/** GET / - List all entities */
		list: function list(_ref2, res) {
			var params = _ref2.params;

			var User = db.user;
			User.findAll({
				order: [['last_name', 'ASC'], ['first_name', 'ASC']],
				include: [User.BusinessGroup, User.Type, User.Role, User.Footprint],
				attributes: attributeFilter
			}).then(function (users) {
				return res.json(users);
			}).catch(function (err) {
				return console.log(err);
			});
		},


		/** POST / - Create a new entity */
		create: function create(_ref3, res) {
			var body = _ref3.body;

			var User = db.user;

			body.u_ur_id = body.role.id;
			body.u_ut_id = body.type.id;

			_bcryptjs2.default.hash(body.password, saltRounds).then(function (hash) {
				body.password = hash;
				return User.create(body);
			}).then(function (user) {
				return user.setGroups(body.groups.map(function (g) {
					return g.id;
				})).then(function () {
					return user.setFootprints(body.footprints.map(function (f) {
						return f.id;
					})).then(function () {
						return User.findByPk(user.id, {
							include: [User.BusinessGroup, User.Type, User.Role, User.Footprint],
							attributes: attributeFilter
						});
					});
				});
			}).then(function (user) {
				return res.json(user);
			}).catch(function (err) {
				return console.log(err);
			});
		},


		/** GET /:id - Return a given entity */
		read: function read(_ref4, res) {
			var user = _ref4.user;

			res.json(user);
		},


		/** PUT /:id - Update a given entity */
		update: function update(_ref5, res) {
			var user = _ref5.user,
			    body = _ref5.body;

			var User = db.user;

			if (body.password) {
				var salt = _bcryptjs2.default.genSaltSync(saltRounds);
				var hash = _bcryptjs2.default.hashSync(body.password, salt);
				body.password = hash;
			}

			db.sequelize.transaction(function (t) {
				return user.update(body, { transaction: t }).then(function () {
					return user.setGroups(body.groups.map(function (g) {
						return g.id;
					}), { transaction: t });
				}).then(function () {
					return user.setFootprints(body.footprints.map(function (g) {
						return g.id;
					}), { transaction: t });
				}).then(function () {
					return user.setType(body.type.id), { transaction: t };
				}).then(function () {
					return user.setRole(body.role.id), { transaction: t };
				});
			}).then(function () {
				return User.findByPk(user.id, { attributes: attributeFilter });
			}).then(function (updated) {
				return res.json(updated);
			}).catch(function (err) {
				return res.status(500).json({ error: err });
			});
		},


		/** DELETE /:id - Delete a given entity */
		delete: function _delete(_ref6, res) {
			var user = _ref6.user;

			db.sequelize.transaction(function (t) {
				return user.setGroups([], { transaction: t }).then(function () {
					return user.destroy({ transaction: t });
				});
			}).then(function () {
				return res.sendStatus(204);
			}).catch(function (err) {
				return res.status(500).json({ error: err });
			});
		}
	});
};