'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _express = require('express');

var _config = require('../config/config');

var _config2 = _interopRequireDefault(_config);

var _jwtSimple = require('jwt-simple');

var _jwtSimple2 = _interopRequireDefault(_jwtSimple);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Retourne une promesse de l'emprise par défaut
 * @param {*} db 
 * @returns 
 */
function getDefaultFootprint(db) {
	var Setting = db.setting;
	return new Promise(function (resolve, reject) {
		Setting.findAll({
			where: {
				s_name: 'osm_footprint'
			}
		}).then(function (res) {
			var setting = res[0];
			if (setting && setting.dataValues && setting.dataValues.value) {
				resolve(setting.dataValues.value);
			} else {
				resolve(null);
			}
		}).catch(function (err) {
			console.error(err);
			reject(err);
		});
	});
}

exports.default = function (_ref) {
	var config = _ref.config,
	    db = _ref.db;


	// Router
	var map = (0, _express.Router)();

	// Modèles
	var Footprint = db.footprint;
	var UserFootprint = db.userFootprint;

	/**
  * Liste des emprises de l'utilisateur
  */
	map.get('/', function (req, res, next) {

		var authorization = req.headers.authorization;
		if (!authorization) {
			return res.status(401).json({ error: 'No credentials sent!' });
		}
		var token = authorization.split(/Bearer\s?/)[1];
		var decoded = _jwtSimple2.default.decode(token, _config2.default.jwt_secret);
		if (!decoded) {
			res.status(401);
		}

		var user_id = decoded.id;
		var filterIds = [];

		// Emprises associées à l'utilisateur
		UserFootprint.findAll({
			where: {
				id: user_id
			}
		}).then(function (userFootprints) {
			for (var i = 0; i < userFootprints.length; i++) {
				if (userFootprints[i].dataValues && userFootprints[i].dataValues.fp_id) {
					filterIds.push(userFootprints[i].dataValues.fp_id);
				}
			}

			// Emprise par défaut
			getDefaultFootprint(db).then(function (defaultFootprintId) {
				if (filterIds.length === 0) {
					filterIds.push(defaultFootprintId);
				}

				// Définition des emprises
				Footprint.findAll({
					order: [['name', 'ASC']],
					where: {
						id: filterIds
					}
				}).then(function (footprints) {
					res.json(footprints);
				}).catch(function (err) {
					return res.status(500).json({ error: err });
				});
			}).catch(function (err) {
				return res.status(500).json({ error: err });
			});
		}).catch(function (err) {
			return res.status(500).json({ error: err });
		});
	});

	return map;
};