'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _resourceRouterMiddleware = require('resource-router-middleware');

var _resourceRouterMiddleware2 = _interopRequireDefault(_resourceRouterMiddleware);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = function (_ref) {
	var config = _ref.config,
	    db = _ref.db;
	return (0, _resourceRouterMiddleware2.default)({

		/** Property name to store preloaded entity on `request`. */
		id: 'wmts',

		/** For requests with an `id`, you can auto-load the entity.
   *  Errors terminate the request, success sets `req[id] = data`.
   */
		load: function load(req, id, callback) {
			var Wmts = db.wmts;

			Wmts.findByPk(id).then(function (wmts) {
				return callback(null, wmts);
			}).catch(function (err) {
				return callback(console.log(err));
			});
		},


		/** GET / - List all entities */
		list: function list(_ref2, res) {
			var params = _ref2.params;

			var Wmts = db.wmts;

			Wmts.findAll({
				order: [['name', 'ASC']]
			}).then(function (wmts) {
				return res.json(wmts);
			}).catch(function (err) {
				return res.status(500).json({ error: err });
			});
		},


		/** POST / - Create a new entity */
		create: function create(_ref3, res) {
			var body = _ref3.body;

			var Wmts = db.wmts;
			Wmts.create(body).then(function (wmts) {
				return res.json(wmts);
			}).catch(function (err) {
				return res.status(500).json({ error: err });
			});
		},


		/** GET /:id - Return a given entity */
		read: function read(_ref4, res) {
			var tag = _ref4.tag;

			res.json(tag);
		},


		/** PUT /:id - Update a given entity */
		update: function update(_ref5, res) {
			var wmts = _ref5.wmts,
			    body = _ref5.body;

			wmts.update(body).then(function () {
				return wmts.reload();
			}).then(function (updated) {
				return res.json(updated);
			}).catch(function (err) {
				return res.status(500).json({ error: err });
			});
		},


		/** DELETE /:id - Delete a given entity */
		delete: function _delete(_ref6, res) {
			var wmts = _ref6.wmts;

			wmts.destroy().then(function () {
				return res.sendStatus(204);
			}).catch(function (err) {
				return res.status(500).json({ error: err });
			});
		}
	});
};