'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * Classe ThemeObjectService
 * 
 * Permet d'effectuer des opérations sur les objets des thèmes
 */
var ThemeObjectService = exports.ThemeObjectService = function () {
    function ThemeObjectService(db, obj, theme, settings) {
        _classCallCheck(this, ThemeObjectService);

        this.db = db;
        this.obj = obj;
        this.theme = theme;
        this.settings = settings;
        this.sequelize = db.sequelize;
        this.Validation = db.validation;
        this.Contributor = db.contributor;
        this.OsmDiff = db.osmdiff;
        this.Footprint = db.footprint;
        this.Op = db.Sequelize.Op;
    }

    /**
     * Retourne l'arbre de diffs
     * @param {*} footprintId 
     * @param {*} filter 
     * @returns 
     */


    _createClass(ThemeObjectService, [{
        key: 'findOsmDiffs',
        value: function findOsmDiffs(footprintId, filter) {
            var _this = this;

            return new Promise(function (resolve, reject) {

                var objectsLimit = _this.settings.objects_limit ? _this.settings.objects_limit : 15;

                // Trouve les identifiants (sert à compter)
                _this.findOsmDiffIds(footprintId, filter).then(function (aIds) {

                    // Trouve les infos en détail pour les x premiers enregistrements
                    var detailIds = aIds.slice(0, objectsLimit);
                    _this.findOsmDiffsDetail(detailIds, filter).then(function (osmdiff) {
                        resolve({
                            osmdiff: osmdiff,
                            nbOsmDiff: aIds.length
                        });
                    });
                });
            });
        }

        /**
         * Retourne la liste des identifiants des osmDiff
         * @param {*} footprintId 
         * @param {*} filter 
         * @returns 
         */

    }, {
        key: 'findOsmDiffIds',
        value: function findOsmDiffIds(footprintId, filter) {
            var _this2 = this;

            return new Promise(function (resolve, reject) {
                var _extends2;

                // requête définissant le groupe d'objets
                var newCompareQuery = _this2.compareQuery('od_new_tags');
                var oldCompareQuery = _this2.compareQuery('od_old_tags');

                // tags définissant le groupe d'objets
                var newCompareTags = _this2.compareTags('od_new_tags');
                var oldCompareTags = _this2.compareTags('od_old_tags');

                var addedRights = [];
                var deletedRights = [];
                var modifiedRights = [];
                var modifiedGeom = [];

                // Prépare la requête WHERE selon les différents cas (vérifier tous les tags ou vérifier certains tags et/ou vérifier la geom)
                if (_this2.VerifAllTagsCase() && _this2.VerifGeomCase()) {
                    console.log('VerifAllTags + VerifGeom');
                    addedRights.push = _this2.sequelize.where(_this2.sequelize.col("od_added_tags"), '!=', '');
                    deletedRights.push = _this2.sequelize.where(_this2.sequelize.col("od_deleted_tags"), '!=', '');
                    modifiedRights.push = _this2.sequelize.where(_this2.sequelize.col("od_modified_tags"), '!=', '');
                    modifiedGeom.push = _this2.sequelize.where(_this2.sequelize.col("od_changed_geom"), '=', 'true');
                } else if (_this2.VerifAllTagsCase() && !_this2.VerifGeomCase()) {
                    console.log('VerifAllTags');
                    addedRights.push = _this2.sequelize.where(_this2.sequelize.col("od_added_tags"), '!=', '');
                    deletedRights.push = _this2.sequelize.where(_this2.sequelize.col("od_deleted_tags"), '!=', '');
                    modifiedRights.push = _this2.sequelize.where(_this2.sequelize.col("od_modified_tags"), '!=', '');
                } else if (!_this2.VerifAllTagsCase() && _this2.VerifTagsToValidate().length && _this2.VerifGeomCase()) {
                    console.log('VerifTags + VerifGeom');
                    addedRights = _this2.compareRights('od_added_tags');
                    deletedRights = _this2.compareRights('od_deleted_tags');
                    modifiedRights = _this2.compareRights('od_modified_tags');
                    modifiedGeom.push = _this2.sequelize.where(_this2.sequelize.col("od_changed_geom"), '=', 'true');
                } else if (!_this2.VerifAllTagsCase() && _this2.VerifTagsToValidate().length && !_this2.VerifGeomCase()) {
                    console.log('VerifTags');
                    addedRights = _this2.compareRights('od_added_tags');
                    deletedRights = _this2.compareRights('od_deleted_tags');
                    modifiedRights = _this2.compareRights('od_modified_tags');
                } else {
                    console.log('Cas autre - id groupe :', _this2.obj.id);
                }

                // N'est plus utilisée car footprintIntersect en direct est trop lent
                //let intersectQuery = [];
                //if (footprintId) {
                //    intersectQuery.push(this.footprintIntersect(footprintId));
                //}

                // Partie WHERE pour l'emprise
                var intersectQuery = [];
                if (footprintId !== 'none') {
                    intersectQuery.push(_this2.sequelize.where(_this2.sequelize.col("footprints.fp_id"), '=', footprintId));
                }

                // Clauses WHERE de la requête
                var whereClause = _extends((_extends2 = {}, _defineProperty(_extends2, _this2.Op.and, [
                //...(footprintId !== 'none' ? {
                //...intersectQuery
                // Clause WHERE sur l'emprise contenant l'osmdiff
                //'$footprints.fp_id$': footprintId,
                //} : {}),
                _defineProperty({}, _this2.Op.or, [_extends({}, newCompareQuery), _extends({}, oldCompareQuery)]), _defineProperty({}, _this2.Op.or, [_extends({}, newCompareTags), _extends({}, oldCompareTags)]), _defineProperty({}, _this2.Op.or, [_extends({}, addedRights), _extends({}, deletedRights), _extends({}, modifiedRights), _extends({}, modifiedGeom)])]), _defineProperty(_extends2, '$status.st_name$', _defineProperty({}, _this2.Op.or, [_defineProperty({}, _this2.Op.in, ['tovalidate', 'waiting']), _defineProperty({}, _this2.Op.eq, null)])), _defineProperty(_extends2, '$validations.val_od_version$', _defineProperty({}, _this2.Op.or, [_defineProperty({}, _this2.Op.eq, _this2.sequelize.col('osmdiff.od_new_version')), _defineProperty({}, _this2.Op.eq, null)])), _extends2), intersectQuery);

                // S'il y a filtre alors il faut joindre plus de tables (ce qui rend la requête plus longue)
                var haveFilter = false;
                if (filter && Object.keys(filter) && Object.keys(filter).length > 0) {
                    haveFilter = true;
                }

                // Jointures de la requête
                var includes = haveFilter ? _this2.getCompleteIncludes() : _this2.getSimpleIncludes(footprintId);

                // Attributs
                var attributes = haveFilter ? null : ['id', 'name', 'new_tags', 'old_tags', 'added_tags', 'deleted_tags', 'modified_tags'];

                _this2.requestOsmDiffs(whereClause, includes, attributes, filter).then(function (diffs) {
                    var aIds = [];
                    for (var i = 0; i < diffs.length; i++) {
                        var diff = diffs[i];
                        aIds.push(diff.id);
                    }
                    resolve(aIds);
                });
            });
        }

        /**
         * Trouve l'ensemble des infos des osmdiffs à partir d'une liste d'identifiants
         * @param {*} aIds 
         * @param {*} filter 
         * @returns 
         */

    }, {
        key: 'findOsmDiffsDetail',
        value: function findOsmDiffsDetail(aIds, filter) {
            var _this3 = this;

            return new Promise(function (resolve, reject) {

                // Clause where de la requête
                var whereClause = {
                    id: aIds

                    // Jointures de la requête
                };var includes = _this3.getCompleteIncludes();

                // Attributs
                var attributes = undefined;

                _this3.requestOsmDiffs(whereClause, includes, attributes, null).then(function (diffs) {
                    resolve(diffs);
                });
            });
        }

        /**
         * Fonction générique pour requêter les osmDiffs
         * @param {*} whereClause 
         * @param {*} includes 
         * @param {*} attributes 
         * @param {*} filter 
         * @returns 
         */

    }, {
        key: 'requestOsmDiffs',
        value: function requestOsmDiffs(whereClause, includes, attributes, filter) {
            var _this4 = this;

            return new Promise(function (resolve, reject) {

                _this4.db.osmdiff.findAll({
                    where: whereClause,
                    order: [['name', 'ASC'], ['id', 'ASC']],
                    include: includes,
                    attributes: attributes
                }).then(function (osmdiff) {
                    if (filter) {
                        resolve(_this4.filterOsmDiffs(osmdiff, filter));
                    } else {
                        resolve(osmdiff);
                    }
                }).catch(function (err) {
                    reject(err);
                });
            });
        }

        /**
         * Applique le filtre sur les osmdiffs
         * @param {*} osmdiff 
         * @param {*} filter 
         * @returns 
         */

    }, {
        key: 'filterOsmDiffs',
        value: function filterOsmDiffs(osmdiff, filter) {
            var _this5 = this;

            if (filter && Object.keys(filter) && Object.keys(filter).length > 0) {
                osmdiff = osmdiff.filter(function (o) {
                    // filter by validations for the associated rights
                    // tous les tags modifiés de l'objet
                    var osmKeys = Object.keys(_extends({}, o.added_tags, o.deleted_tags, o.modified_tags));
                    // On y ajoute le tag virtuel "lbt_geom"
                    osmKeys.push('lbt_geom');

                    // validations are valid only because osm object is to validate
                    // tags validés
                    var validatedKeys = o.validations.filter(function (v) {
                        return v.status.name === 'valid';
                    }).reduce(function (a, v) {
                        return [].concat(a, v.tags);
                    }, []);

                    // Si tous les tags sont à valider (case cochée dans les droits), on met tous les tags ajoutés, modifiés, supprimés à la liste des tags à valider depuis les droits
                    // Sinon ce sont les tags précisés dans les droits de validation
                    var rightKeys = void 0;
                    var verif_all_tags = _this5.obj.rights.reduce(function (a, r) {
                        return [].concat(a, r.r_verif_all_tags);
                    }, []);
                    if (verif_all_tags.includes(true)) {
                        rightKeys = Object.keys(_extends({}, o.added_tags, o.deleted_tags, o.modified_tags));
                    } else {
                        // tags à valider depuis les droits
                        rightKeys = _this5.obj.rights.reduce(function (a, r) {
                            return [].concat(a, r.keys);
                        }, []).map(function (k) {
                            return k.name;
                        });
                    }

                    // Si la géométrie est à valider (case cochée dans les droits) et que la géométrie de l'objet a été modifiée, on ajoute le tag virtuel "lbt_geom" à la liste des tags à valider depuis les droits
                    if (o.changed_geom) {
                        var verif_geom = _this5.obj.rights.reduce(function (a, r) {
                            return [].concat(a, r.r_verif_geom);
                        }, []);
                        if (verif_geom.includes(true)) {
                            rightKeys.push('lbt_geom');
                        }
                    }

                    // find keys to validate i.e. osm keys that exist in right
                    // tags à valider : différence entre les tags présents modifiés et les tags à valider
                    var keysToValidate = osmKeys.filter(function (k) {
                        return rightKeys.includes(k);
                    });

                    // find remaining keys to validate i.e. keys that are not valid yet
                    // tags qui restent à valider
                    var remainingKeys = keysToValidate.filter(function (k) {
                        return !validatedKeys.includes(k);
                    });

                    // is there some keys?
                    // S'il reste des tags à valider, affiche l'objet
                    return !!remainingKeys.length;
                })
                // Filtre passé par l'URL
                .filter(function (osm) {
                    try {
                        filter = filter ? filter : {};
                        filter.contributors = filter.contributors ? filter.contributors : [];
                        filter.vigilance = filter.vigilance ? filter.vigilance : {};
                        return (!filter.contributors.length || osm.contributor && filter.contributors.some(function (c) {
                            return c.id === osm.contributor.id;
                        })) && (!filter.vigilance.id || osm.contributor && osm.contributor.vigilance && filter.vigilance.id === osm.contributor.vigilance.id);
                    } catch (e) {
                        return true;
                    }
                });
            }

            return osmdiff;
        }

        /**
         * Retourne les inclusions simples (mais rapides) pour requestOsmDiffs
         * @returns
         */

    }, {
        key: 'getSimpleIncludes',
        value: function getSimpleIncludes(footprintId) {
            return [{
                association: this.OsmDiff.Status,
                required: false,
                attributes: ['st_name']
            }, {
                association: this.OsmDiff.Validation,
                required: false,
                attributes: ['val_od_version']
            }, {
                association: this.OsmDiff.Footprint,
                attributes: ['fp_id']
            }];
        }

        /**
         * Retourne les inclusions complètes (mais lentes) pour requestOsmDiffs (jointures)
         * @returns 
         */

    }, {
        key: 'getCompleteIncludes',
        value: function getCompleteIncludes() {
            return [{
                association: this.OsmDiff.NewChangeset
                // attributes: ['ocs_id'],
            }, {
                association: this.OsmDiff.Status,
                required: false
                // attributes: ['st_name'],
            }, {
                association: this.OsmDiff.Validation,
                required: false,
                include: [{
                    association: this.Validation.Status
                }, {
                    association: this.Validation.User
                    // attributes: ['u_id', 'u_login', 'u_first_name', 'u_last_name'],
                }]
            },
            // Récupère l'id fp_id de l'emprise associée (jointure 1-n : plusieurs emprises possibles)
            {
                association: this.OsmDiff.Footprint,
                attributes: ['fp_id']
            }, {
                association: this.OsmDiff.Contributor,
                // attributes: ['ct_name'],
                include: [{
                    association: this.Contributor.Vigilance
                    // attributes: ['vg_color'],
                }]
            }];
        }

        /**
         * fonction de génération clause WHERE colonne contient (@>) hstore
         * @param {*} col 
         * @param {*} hArray 
         * @returns 
         */

    }, {
        key: 'hstoreContainHstore',
        value: function hstoreContainHstore(col, hArray) {
            return this.sequelize.where(this.sequelize.col(col), '@>', this.sequelize.fn('hstore', this.sequelize.literal('ARRAY[\'' + hArray.join('\',\'') + '\']')));
        }

        /**
         * fonction de génération clause WHERE colonne - opérateur - clés
         * @param {*} col 
         * @param {*} op 
         * @param {*} keys 
         * @returns 
         */

    }, {
        key: 'hstoreContainKeys',
        value: function hstoreContainKeys(col, op, keys) {
            return this.sequelize.where(this.sequelize.col(col), op, this.sequelize.literal('ARRAY[\'' + keys.join('\',\'') + '\']'));
        }

        /**
         * Traduit une requête JSON en requête sequelize
         * @param {*} col 
         * @param {*} op 
         * @param {*} keys 
         * @returns 
         */

    }, {
        key: 'translateQuery',
        value: function translateQuery(col, parsedQuery) {

            var firstOp = this.getFirstOperator(parsedQuery);
            if (Array.isArray(parsedQuery[firstOp]) && firstOp !== null) {
                var conditions = [];

                for (var i = 0; i < parsedQuery[firstOp].length; i++) {
                    var node = parsedQuery[firstOp][i];
                    var key = Object.keys(node)[0];
                    var val = node[key];

                    if (typeof val === 'string') {
                        if (val === '*') {
                            conditions.push(this.sequelize.where(this.sequelize.col(col), '?', '' + key));
                        } else {
                            conditions.push(this.sequelize.where(this.sequelize.col(col), '@>', '"' + key + '"=>"' + val + '"'));
                        }
                    } else if ((typeof val === 'undefined' ? 'undefined' : _typeof(val)) === 'object') {

                        // Opérateur NOT
                        if ('[op.ne]' in val) {
                            var opVal = val['[op.ne]'];
                            var testSQL = 'not ' + col + ' @> \'' + ('"' + key + '"=>"' + opVal + '"') + '\'';
                            conditions.push(this.sequelize.literal('(' + testSQL + ')'));

                            // Les opérateurs > < >= <= ne peuvent pas être utilisés avec du HSTORE, ils sont donc ignorés
                        } else if ('[op.gt]' in val) {
                            var _opVal = val['[op.gt]'];
                        } else if ('[op.gte]' in val) {
                            var _opVal2 = val['[op.gte]'];
                        } else if ('[op.lt]' in val) {
                            var _opVal3 = val['[op.lt]'];
                        } else if ('[op.lte]' in val) {
                            var _opVal4 = val['[op.lte]'];
                        } else {
                            conditions.push(this.translateQuery(col, node));
                        }
                    }
                }

                if (firstOp === '[op.or]') {
                    return _defineProperty({}, this.Op.or, conditions);
                } else if (firstOp === '[op.and]') {
                    return _defineProperty({}, this.Op.and, conditions);
                }
            }
        }

        /**
         * Retourne le premier opérateur présent
         * @param {*} col 
         * @param {*} op 
         * @param {*} keys 
         * @returns 
         */

    }, {
        key: 'getFirstOperator',
        value: function getFirstOperator(parsedQuery) {
            if (parsedQuery['[op.or]']) {
                return '[op.or]';
            } else if (parsedQuery['[op.and]']) {
                return '[op.and]';
            } else {
                return false;
            }
        }

        /**
         * clause WHERE col / requête du groupe d'objets (og_query)
         * @param {*} col 
         * @returns 
         */

    }, {
        key: 'compareQuery',
        value: function compareQuery(col) {
            var clauses = [];
            // Requête groupe d'objets
            if (this.obj.query_json) {
                var query = this.translateQuery(col, this.obj.query_json);
                clauses.push(query);
            }
            // Si query est vide, on crée une clause WHERE toujours vraie à la place de hstoreContainHstore
            else {
                    clauses.push(this.sequelize.where(this.sequelize.col(col), '=', this.sequelize.col(col)));
                }

            // clauses renvoie une clause WHERE pour la future requête
            return clauses;
        }

        /**
         * fonction de comparaison avec les tags obligatoires du groupe d'objets (og_tags)
         * @param {*} col 
         * @returns 
         */

    }, {
        key: 'compareTags',
        value: function compareTags(col) {
            var clauses = [];

            if (this.obj.keys.length) {
                var keys = this.obj.keys.map(function (k) {
                    return k.name;
                });
                clauses.push(this.hstoreContainKeys(col, '?&', keys));
            }
            // Si keys est vide, on crée une clause WHERE toujours vraie à la place de hstoreContainKeys
            else {
                    clauses.push(this.sequelize.where(this.sequelize.col(col), '=', this.sequelize.col(col)));
                }

            return clauses;
        }

        /**
         * fonction de comparaison avec les tags à valider (droits)
         * @param {*} col 
         * @returns 
         */

    }, {
        key: 'compareRights',
        value: function compareRights(col) {
            var clauses = [];
            if (this.obj.rights.length) {
                var keys = this.obj.rights.reduce(function (a, r) {
                    return [].concat(a, r.keys);
                }, []).map(function (k) {
                    return k.name;
                });
                clauses.push(this.hstoreContainKeys(col, '?|', keys));
            }
            return clauses;
        }

        /**
         * Est-ce que la case "Valider les objets dont n'importe quel attribut a été modifié" est cochée (r_verif_all_tags) ?
         * @param {*} col 
         * @returns 
         */

    }, {
        key: 'VerifAllTagsCase',
        value: function VerifAllTagsCase() {
            var result = false;
            if (this.obj.rights.length) {
                var verif_all_tags = this.obj.rights.reduce(function (a, r) {
                    return [].concat(a, r.r_verif_all_tags);
                }, []);
                if (verif_all_tags.includes(true)) {
                    result = true;
                }
            }
            return result;
        }

        /**
         * Est-ce que la case "Valider les objets dont la géométrie a changé" est cochée (r_verif_geom) ?
         * @param {*} col 
         * @returns 
         */

    }, {
        key: 'VerifGeomCase',
        value: function VerifGeomCase() {
            var result = false;
            if (this.obj.rights.length) {
                var verif_geom = this.obj.rights.reduce(function (a, r) {
                    return [].concat(a, r.r_verif_geom);
                }, []);
                if (verif_geom.includes(true)) {
                    result = true;
                }
            }
            return result;
        }

        /**
         * Est-ce que "Valider les objets dont les attributs suivants ont été modifiés" contient des attributs ?
         * @param {*} col 
         * @returns 
         */

    }, {
        key: 'VerifSelectedTags',
        value: function VerifSelectedTags() {
            var result = false;
            if (this.obj.rights.length) {
                var keys = this.obj.rights.reduce(function (a, r) {
                    return [].concat(a, r.keys);
                }, []).map(function (k) {
                    return k.name;
                });
                if (keys.length) {
                    result = true;
                }
            }
            return result;
        }

        /**
         *  Quels sont les attributs à valider ?
         * @param {*} col 
         * @returns 
         */

    }, {
        key: 'VerifTagsToValidate',
        value: function VerifTagsToValidate() {
            var tagsToValidate = [];
            if (this.obj.rights.length) {
                var keys = this.obj.rights.reduce(function (a, r) {
                    return [].concat(a, r.keys);
                }, []).map(function (k) {
                    return k.name;
                });
                tagsToValidate.push(keys);
            }
            return tagsToValidate;
        }

        // N'est plus utilisée (st_intersects à la volée trop long)
        /**
         * Intersection par emprise
         * @param {*} footprintId 
         * @returns 
         */

    }, {
        key: 'footprintIntersect',
        value: function footprintIntersect(footprintId) {
            var tempSQL = this.sequelize.dialect.QueryGenerator.selectQuery('lebontag.lbt_footprint', {
                // ST_CollectionExtract(geom,3) convertit les multicollections en multipolygons
                attributes: [this.sequelize.literal('(st_dump(ST_CollectionExtract(lbt_footprint.fp_geom,3))).geom as fp_geom')],
                where: {
                    fp_id: footprintId
                }
                // limit inutile car req basée sur ID unique de l'emprise
                //limit: 1,
            }).slice(0, -1).replace(/"lebontag.lbt_footprint"/g, '"lebontag"."lbt_footprint"');

            return this.sequelize.where(this.sequelize.fn('ST_Intersects', this.sequelize.col('od_new_geom'), this.sequelize.literal('(' + tempSQL + ')')), true);
        }
    }]);

    return ThemeObjectService;
}();